from qt import *
from Qwt4 import *
from numpy import *
from threading import *
import string,os,time

from windowbase import cvForm

def read_cfg_file(filename):
	f=open(filename,'r')
	a=f.readlines()
	param={}
	for i in a:
		b=string.split(i.strip(),'=')
		param[b[0]]=b[1]
	return param

class MeasureThread(Thread):

	def __init__(self, wid, dirname, *args):
		self.w=wid
		self.dirname=dirname
		apply(Thread.__init__, (self, ) + args)

	def run(self):
		par=read_cfg_file('cv.cfg')
		Vb_start=float(par['VStart'])
		Vb_step=float(par['VStep'])
		Vb_stop=float(par['VStop'])
		f1=float(par['Freq1'])
		f2=float(par['Freq2'])
		nfreq=int(par['NFreq'])
		spect=par['Spectrum']
		N=int((Vb_stop-Vb_start)/Vb_step+1)
		results=zeros((N,10),Float)	# Number of points * (v_up,c1,g1,c2,g2,v_down,c1,g1,c2,g2)
		msg=[nfreq,spect,results]
		for i in range(N):
			results[i,0]=float(i)
			results[i,1]=float(i)
			self.w.queue.put(msg)
			time.sleep(0.1)
		self.w.is_running=1

class Window (cvForm):
	def __init__(self):
		cvForm.__init__(self)
		if os.path.isfile('cv.cfg'):
			self.is_running=1	#1 is running but not measuring, 2 is running and measuring,0 stop but wait for measurement, -1 stop immediately
			#Initialise the layout
			#Read parameters from a config file if it exists.
			self.nfreq=1
			param=read_cfg_file('cv.cfg')
			self.datadirLE.setText(os.getcwd())
			self.batchnameLE.setText(param['Batch'])
			self.samplenameLE.setText(param['Sample'])
			self.dopingCB.setCurrentText (param['Doping'])
			self.paddiamLE.setText(param['Dp'])
			self.commentsTE.setText(param['Comment'])
			self.vstartLE.setText(param['VStart'])
			self.vstepLE.setText(param['VStep'])
			self.vstopLE.setText(param['VStop'])
			self.polarityCB.setCurrentText (param['Polarity'])
			self.nfreqCB.setCurrentText(param['NFreq'])
			self.freq1LE.setText(param['Freq1'])
			self.freq2LE.setText(param['Freq2'])
			self.voscLE.setText(param['VOsc'])
			self.twait1LE.setText(param['TWaitStart'])
			self.spectrumCB.setCurrentText(param['Spectrum'])
			self.twait2LE.setText(param['TWaitSwitch'])
			self.integtCB.setCurrentText(param['Integtime'])
	
		#Set the initialise the graphs
		self.capacitancePL.setAutoLegend(1)
		self.capacitancePL.setLegendPos(Qwt.Right)
		self.capacitancePL.setAxisTitle(QwtPlot.xBottom, 'V [V]')
		self.capacitancePL.setAxisTitle(QwtPlot.yLeft, 'C [F]')

		self.conductancePL.setAutoLegend(1)
		self.conductancePL.setLegendPos(Qwt.Right)
		self.conductancePL.setAxisTitle(QwtPlot.xBottom, 'V [V]')
		self.conductancePL.setAxisTitle(QwtPlot.yLeft, 'G [S]')

	def stopmeasureSlot(self):
		self.is_running=1

	def startmeasSlot(self):
		if (self.is_running==1):
			self.is_running=2
			f=open('cv.cfg','w')
			f.write('Batch=%s\n'      % self.batchnameLE.text())
			f.write('Sample=%s\n'     % self.samplenameLE.text())
			f.write('Doping=%s\n'     % self.dopingCB.currentText())
			f.write('Dp=%s\n'         % self.paddiamLE.text())
			f.write('Comment=%s\n'    % self.commentsTE.text())
			f.write('VStart=%s\n'     % self.vstartLE.text())
			f.write('VStep=%s\n'      % self.vstepLE.text())
			f.write('VStop=%s\n'      % self.vstopLE.text())
			f.write('Polarity=%s\n'   % self.polarityCB.currentText())
			f.write('NFreq=%s\n'      % self.nfreqCB.currentText())
			f.write('Freq1=%s\n'      % self.freq1LE.text())
			f.write('Freq2=%s\n'      % self.freq2LE.text())
			f.write('VOsc=%s\n'       % self.voscLE.text())
			f.write('Integtime=%s\n'  % self.integtCB.currentText())
			f.write('TWaitStart=%s\n' % self.twait1LE.text())
			f.write('Spectrum=%s\n'   % self.spectrumCB.currentText())
			f.write('TWaitSwitch=%s\n'% self.twait2LE.text())
			f.flush()
			nfreq=int("%s"%self.nfreqCB.currentText())
			self.capacitancePL.clear()
			self.conductancePL.clear()
			self.cvf1=self.capacitancePL.insertCurve('f1s1', QwtPlot.xBottom, QwtPlot.yLeft)
			self.capacitancePL.setCurvePen(self.cvf1, QPen(Qt.red))
			self.gvf1=self.conductancePL.insertCurve('f1s1', QwtPlot.xBottom, QwtPlot.yLeft)
			self.conductancePL.setCurvePen(self.gvf1, QPen(Qt.red))
			cvf2=0
			gvf2=0
			if nfreq==2:
				self.cvf2=self.capacitancePL.insertCurve('f2s1',QwtPlot.xBottom,QwtPlot.yLeft)
				self.capacitancePL.setCurvePen(self.cvf2, QPen(Qt.blue))
				self.gvf2=self.conductancePL.insertCurve('f2s1',QwtPlot.xBottom,QwtPlot.yRight)
				self.conductancePL.setCurvePen(self.gvf2, QPen(Qt.blue))

			self.cvf3=self.capacitancePL.insertCurve('f1s2', QwtPlot.xBottom, QwtPlot.yLeft)
			self.capacitancePL.setCurvePen(self.cvf3, QPen(Qt.green))
			self.gvf3=self.conductancePL.insertCurve('f1s2', QwtPlot.xBottom, QwtPlot.yLeft)
			self.conductancePL.setCurvePen(self.gvf3, QPen(Qt.green))
			cvf4=0
			gvf4=0
			if nfreq==2:
				self.cvf4=self.capacitancePL.insertCurve('f2s2',QwtPlot.xBottom,QwtPlot.yLeft)
				self.capacitancePL.setCurvePen(self.cvf4, QPen(Qt.black))
				self.gvf4=self.conductancePL.insertCurve('f2s2',QwtPlot.xBottom,QwtPlot.yRight)
				self.conductancePL.setCurvePen(self.gvf4, QPen(Qt.black))
			MeasureThread(self,self.datadirLE.text()).start()

	def exitSlot(self):
		if self.is_running==2:
			self.is_running=0
		if self.is_running==1:
			self.is_running=-1

	def browseSlot(self):
		fn = QFileDialog.getExistingDirectory(QString.null,self)
		self.datadirLE.setText(str(fn))
